#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
#                                                                          #
#  Utilitaire de configuration du support pour bacula                      #
#                                                                          #
############################################################################

from optparse import OptionParser, OptionGroup
import sys

from pyeole.bacula import save_bacula_support_usb, save_bacula_support_smb, \
        save_bacula_support_manual, save_bacula_mail, load_bacula_mail,\
        load_bacula_support, apply_bacula_support, apply_bacula_jobs, \
        apply_bacula_mail, run_bacula_cmd, display_bacula_jobs, add_job, \
        run_backup, Disabled
from creole.config import LOCK_PATH, LOCK_FILES
from pyeole.lock import add_lock, del_lock, check_lock

lock_name = 'eolesauvegarde'

def main():
    def conf_support():
        support = options.support
        usb_path = options.usb_path
        smb_machine = options.smb_machine
        smb_ip = options.smb_ip
        smb_partage = options.smb_partage
        smb_login = options.smb_login
        smb_password = options.smb_password
        try:
            if support == 'usb':
                save_bacula_support_usb(usb_path=usb_path)
            elif support == 'smb':
                save_bacula_support_usb(smb_machine=smb_machine,
                    smb_ip=smb_ip, smb_partage=smb_partage,
                    smb_login=smb_login, smb_password=smb_password)
            elif support == 'manual':
                save_bacula_support_manual()
        except Disabled:
            pass
        except Exception, e:
            print e
            sys.exit(1)

    def conf_mail():
        try:
            save_bacula_mail(mail_ok=options.mail_ok, mail_error=options.mail_error)
        except Exception, e:
            print e
            sys.exit(1)

    def conf_job():
        try:
            add_job(job=options.job, level=options.job_level, day=options.job_day,
                    hour=options.job_hour, end_day=options.job_end_day)
        except Exception, e:
            print e
            sys.exit(1)

    def display_infos():
        begin = "Support : {0}"
        try:
            ret = str(load_bacula_support())
            print begin.format(ret)
        except Disabled:
            pass
        except Exception, e:
            print e
            sys.exit(1)

        begin = "Mail : {0}"
        try:
            ret = load_bacula_mail()
            if ret == False:
                print begin.format('non configuré')
            else:
                print begin.format(str(ret))
        except Disabled:
            pass
        except Exception, e:
            print e
            sys.exit(1)

        msg = "Programmation : "
        try:
            jobs = display_bacula_jobs()
            if jobs == None:
                print msg + 'non configuré'
            else:
                print msg
                for num, job in jobs:
                    print "        {0}".format(str(job))
        except Disabled:
            pass
        except Exception, e:
            import traceback
            traceback.print_exc()
            print e
            sys.exit(1)

    def apply_conf():
        try:
            apply_bacula_support(load=False)
        except Disabled:
            pass
        except Exception, e:
            print e
            sys.exit(1)

        try:
            apply_bacula_mail(load=False)
        except Disabled:
            pass
        except Exception, e:
            print e
            sys.exit(1)

        try:
            apply_bacula_jobs(load=False)
        except Disabled:
            pass
        except Exception, e:
            print e
            sys.exit(1)

        try:
            run_bacula_cmd('reload')
        except Exception, e:
            print e
            sys.exit(1)

    def run_back():
        try:
            print run_backup(options.level)
        except Exception, e:
            print e
            sys.exit(1)

    def check_all_locks():
        """Test d'existance de l'un des locks
        du système
        """
        for lock_file in LOCK_FILES:
            if check_lock(LOCK_PATH, lock_file):
                return lock_file
        return ''


    #get options
    parser = OptionParser(usage="Seulement une option \n%prog [option]")
    parser.add_option("-d", "--display", action="store_true", dest="display",
            default=False, help=u"Liste les configurations")
    parser.add_option("-a", "--apply", action="store_true", dest="apply_c",
            default=False, help=u"Applique les configurations")
    group = OptionGroup(parser, "Configuration du support de sauvegarde")
    group.add_option("-s", "--support", dest="support", help=u"Type du support de sauvegarde (usb, smb ou manual)")
    group.add_option("--usb_path", dest="usb_path", help=u"Support usb : chemin du disque usb")
    group.add_option("--smb_machine", dest="smb_machine", help=u"Support smb : nom machine avec le partage")
    group.add_option("--smb_ip", dest="smb_ip", help=u"Support smb : ip de la machine avec le partage")
    group.add_option("--smb_partage", dest="smb_partage", help=u"Support smb : nom du partage")
    group.add_option("--smb_login", dest="smb_login", help=u"Support smb : nom de l'utilisateur smb (facultatif)")
    group.add_option("--smb_password", dest="smb_password", help=u"Support smb : mot de passe smb (facultatif)")
    parser.add_option_group(group)
    group2 = OptionGroup(parser, "Configuration des adresses mails")
    group2.add_option("-m", "--mail", dest="mail", action="store_true", default=False,
            help=u"Configuration des adresses mails de sauvegarde (sans autre option, suppression)")
    group2.add_option("--mail_ok", dest="mail_ok", help=u"    Mail admin sauvegarde (facultatif)")
    group2.add_option("--mail_error", dest="mail_error", help=u"    Mail admin sauvegarde pour les erreurs (facultatif)")
    parser.add_option_group(group2)
    group3 = OptionGroup(parser, "Programmation des sauvegardes")
    group3.add_option("-j", "--job", dest="job", help=u"Type de programmation (daily, weekly ou monthly)")
    group3.add_option("--job_level", dest="job_level", help=u"Niveau de programmation (Full, Incremental ou Differential)")
    group3.add_option("--job_day", dest="job_day", help=u"Jour de programmation (1 pour la nuit de dimanche au lundi, ..., 7 samedi au dimanche). La nuit comment à 12 h")
    group3.add_option("--job_end_day", dest="job_end_day", help=u"Si programmation daily, nuit de la fin de la page")
    group3.add_option("--job_hour", dest="job_hour", help=u"Heure de programmation (chiffre de 0 à 23), la nuit commence à 12h et fini à 11h le lendemain")
    parser.add_option_group(group3)
    group4 = OptionGroup(parser, "Lancer une sauvegarde")
    group4.add_option("-n", "--now", dest="now", help=u"Lancer une sauvegarde immédiate", action="store_true", default=False)
    group4.add_option("--level", dest="level", help=u"Niveau de sauvegarde (Full, Incremental ou Differential)")
    parser.add_option_group(group4)
    group5 = OptionGroup(parser, "Gestion des verrous")
    group5.add_option("--lock", dest="lock", help=u"Mettre un verrou de sauvegarde", action="store_true", default=False)
    group5.add_option("--unlock", dest="unlock", help=u"Supprimer le verrou de sauvegarde", action="store_true", default=False)
    parser.add_option_group(group5)


    (options, args) = parser.parse_args()


    if options.support == None and options.mail == False and options.job == None \
            and options.display == False and options.apply_c == False \
            and options.now == False and options.lock == False \
            and options.unlock == False:
        parser.print_help()
        sys.exit(1)

    if options.support != None:
        conf_support()

    if options.mail != False:
        conf_mail()

    if options.job != None:
        conf_job()

    if options.display != False:
        display_infos()

    if options.apply_c != False:
        apply_conf()

    if options.now != False:
        run_back()

    if options.lock != False:
        if check_lock(LOCK_PATH, lock_name):
            print "Verrou de sauvegarde déjà placé"
            sys.exit(1)
        if check_all_locks():
            print "Un verrou autre que celui de la sauvegarde est placé"
            sys.exit(1)
        return add_lock(LOCK_PATH, lock_name)

    if options.unlock != False:
        if not check_lock(LOCK_PATH, lock_name):
            print "Verrou de sauvegarde inexistant"
            sys.exit(1)
        return del_lock(LOCK_PATH, lock_name)

if __name__ == "__main__":
    main()

